@@ -12,10 +12,13 @@ behavior and easy switching between providers.
 """
 
 from __future__ import annotations
+from typing import Self
 
-from agentle.generations.models.generation.trace_params import TraceParams
 from rsb.models.base_model import BaseModel
 from rsb.models.field import Field
+from rsb.models.model_validator import model_validator
+
+from agentle.generations.models.generation.trace_params import TraceParams
 
 
 class GenerationConfig(BaseModel):
@@ -35,7 +38,6 @@ class GenerationConfig(BaseModel):
         top_p: Nucleus sampling parameter - considers only the top p% of probability mass.
             Range 0-1.
         top_k: Only sample from the top k tokens at each step.
-        google_generation_kwargs: Additional parameters specific to Google AI models.
         trace_params: Parameters for tracing the generation for observability.
         timeout: Maximum time in seconds to wait for a generation before timing out.
     """
@@ -72,21 +74,47 @@ class GenerationConfig(BaseModel):
         ge=0.0,
         examples=[10, 40, 100],
     )
-    google_generation_kwargs: dict[str, object] | None = Field(
-        default=None,
-        description="Additional parameters specific to Google AI model generation. Allows passing provider-specific parameters that aren't standardized across all providers in the Agentle framework.",
-    )
+
     trace_params: TraceParams = Field(
         default_factory=lambda: TraceParams(),
         description="Configuration for tracing and observability of the generation process. Controls what metadata is captured about the generation for monitoring, debugging, and analysis purposes.",
     )
+
     timeout: float | None = Field(
+        default=None,
+        description="Maximum time in milliseconds to wait for a generation response before timing out. Helps prevent indefinite waits for slow or stuck generations. Recommended to set based on expected model and prompt complexity.",
+        gt=0,
+        examples=[10000, 30000, 60000],
+    )
+
+    timeout_s: float | None = Field(
         default=None,
         description="Maximum time in seconds to wait for a generation response before timing out. Helps prevent indefinite waits for slow or stuck generations. Recommended to set based on expected model and prompt complexity.",
         gt=0,
         examples=[10.0, 30.0, 60.0],
     )
 
+    timeout_m: float | None = Field(
+        default=None,
+        description="Maximum time in minutes to wait for a generation response before timing out. Helps prevent indefinite waits for slow or stuck generations. Recommended to set based on expected model and prompt complexity.",
+        gt=0,
+        examples=[1.0, 3.0, 6.0],
+    )
+
+    @model_validator(mode="after")
+    def validate_timeout(self) -> Self:
+        # check if all timeout fields are set. only one of them should be set.
+        if (
+            self.timeout is not None
+            and self.timeout_s is not None
+            and self.timeout_m is not None
+        ):
+            raise ValueError(
+                "Only one of timeout or timeout_s or timeout_m should be set."
+            )
+
+        return self
+
     def clone(
         self,
         *,
@@ -95,7 +123,6 @@ class GenerationConfig(BaseModel):
         new_n: int | None = None,
         new_top_p: float | None = None,
         new_top_k: float | None = None,
-        new_google_generation_kwargs: dict[str, object] | None = None,
         new_trace_params: TraceParams | None = None,
         new_timeout: float | None = None,
     ) -> GenerationConfig:
@@ -111,13 +138,16 @@ class GenerationConfig(BaseModel):
             new_n: New number of completions value, if provided.
             new_top_p: New top_p value, if provided.
             new_top_k: New top_k value, if provided.
-            new_google_generation_kwargs: New Google-specific parameters, if provided.
             new_trace_params: New trace parameters, if provided.
             new_timeout: New timeout value, if provided.
 
         Returns:
             A new GenerationConfig instance with the specified updates applied.
         """
+
+        if new_trace_params is not None:
+            self.trace_params.update(new_trace_params)
+
         return GenerationConfig(
             temperature=new_temperature
             if new_temperature is not None
@@ -128,12 +158,7 @@ class GenerationConfig(BaseModel):
             n=new_n if new_n is not None else self.n,
             top_p=new_top_p if new_top_p is not None else self.top_p,
             top_k=new_top_k if new_top_k is not None else self.top_k,
-            google_generation_kwargs=new_google_generation_kwargs
-            if new_google_generation_kwargs is not None
-            else self.google_generation_kwargs,
-            trace_params=new_trace_params
-            if new_trace_params is not None
-            else self.trace_params,
+            trace_params=self.trace_params,
             timeout=new_timeout if new_timeout is not None else self.timeout,
         )
 
