@@ -17,18 +17,20 @@ class FilePart(BaseModel):
 
     This class handles binary file data with appropriate MIME type validation.
     """
+
     type: Literal["file"] = Field(
         default="file",
         description="Discriminator field to identify this as a file message part.",
     )
 
-    data: bytes = Field(description="The binary content of the file.")
+    data: bytes | str = Field(
+        description="The binary content of the file. or the Base64 encoded contents"
+    )
 
     mime_type: str = Field(
         description="The MIME type of the file, must be a valid MIME type from Python's mimetypes module."
     )
 
-
     @property
     def text(self) -> str:
         """
@@ -39,6 +41,9 @@ class FilePart(BaseModel):
         """
         return f"<file>\n{self.mime_type}\n </file>"
 
+    def __str__(self) -> str:
+        return self.text
+
     def __post_init__(self) -> None:
         """
         Validates that the provided MIME type is official.
