@@ -22,11 +22,11 @@ class DeveloperMessage(BaseModel):
     text, files, and tools.
     """
 
-    parts: Sequence[TextPart | FilePart | Tool] = Field(
-        description="The sequence of message parts that make up this developer message."
-    )
-
     role: Literal["developer"] = Field(
         default="developer",
         description="Discriminator field to identify this as a developer message. Always set to 'developer'.",
     )
+
+    parts: Sequence[TextPart | FilePart | Tool] = Field(
+        description="The sequence of message parts that make up this developer message.",
+    )
