@@ -41,8 +41,9 @@ await trace_client.end()
 from __future__ import annotations
 
 import abc
+from collections.abc import Mapping
 from datetime import datetime
-from typing import Sequence
+from typing import Any, Sequence
 
 
 class StatefulObservabilityClient(abc.ABC):
@@ -265,9 +266,11 @@ class StatefulObservabilityClient(abc.ABC):
         session_id: str | None = None,
         input: object | None = None,
         output: object | None = None,
-        metadata: dict[str, object] | None = None,
+        metadata: Mapping[str, object] | None = None,
         tags: Sequence[str] | None = None,
         timestamp: datetime | None = None,
+        usage_details: Mapping[str, Any] | None = None,
+        cost_details: Mapping[str, Any] | None = None,
     ) -> StatefulObservabilityClient:
         """
         End the current trace, span, or generation with optional final data.
@@ -329,6 +332,40 @@ class StatefulObservabilityClient(abc.ABC):
         """
         ...
 
+    @abc.abstractmethod
+    async def score_trace(
+        self,
+        *,
+        name: str,
+        value: float | str,
+        trace_id: str | None = None,
+        comment: str | None = None,
+    ) -> None:
+        """
+        Add a score to a trace.
+
+        Scores provide quantitative or categorical assessments of trace quality or performance.
+        They can be used for filtering, sorting, and analysis in the observability platform.
+
+        Args:
+            name: Identifier for the score (e.g., "trace_success", "response_quality").
+            value: The score value, which can be:
+                - A float (typically 0.0-1.0) for numeric scores
+                - A string for categorical scores
+            trace_id: Optional ID of the trace to score. If not provided, uses the current trace.
+            comment: Optional explanation or context for the score.
+
+        Example:
+            ```python
+            # Mark a trace as successful
+            await client.score_trace(name="trace_success", value=1.0, comment="Completed without errors")
+
+            # Categorize a trace
+            await client.score_trace(name="response_type", value="detailed", comment="Comprehensive answer")
+            ```
+        """
+        ...
+
     # Helper methods that build on the abstract methods
 
     async def model_generation(
@@ -372,6 +409,8 @@ class StatefulObservabilityClient(abc.ABC):
         output: dict[str, object],
         start_time: datetime | None = None,
         metadata: dict[str, object] | None = None,
+        usage_details: dict[str, object] | None = None,
+        cost_details: dict[str, object] | None = None,
     ) -> StatefulObservabilityClient:
         """
         End the current trace/span/generation with success status and timing.
@@ -380,6 +419,8 @@ class StatefulObservabilityClient(abc.ABC):
             output: The output data
             start_time: Start time for calculating latency
             metadata: Additional metadata
+            usage_details: Usage information (tokens, etc.)
+            cost_details: Cost information
 
         Returns:
             The parent stateful client
@@ -387,14 +428,29 @@ class StatefulObservabilityClient(abc.ABC):
         complete_metadata: dict[str, object] = {"status": "success"}
 
         if start_time:
-            # Convert float to object type for dictionary
             latency_ms = (datetime.now() - start_time).total_seconds() * 1000
             complete_metadata["latency_ms"] = latency_ms
 
         if metadata:
             complete_metadata.update(metadata)
 
-        return await self.end(output=output, metadata=complete_metadata)
+        # Make sure that cost_details contains proper field names
+        if cost_details and "total" in cost_details:
+            # Ensure we have all necessary fields with consistent naming
+            standardized_cost = {
+                "input": cost_details.get("input", 0.0),
+                "output": cost_details.get("output", 0.0),
+                "total": cost_details.get("total", 0.0),
+                "currency": cost_details.get("currency", "USD"),
+            }
+            cost_details = standardized_cost
+
+        return await self.end(
+            output=output,
+            metadata=complete_metadata,
+            usage_details=usage_details,
+            cost_details=cost_details,
+        )
 
     async def complete_with_error(
         self,
