@@ -54,8 +54,9 @@ from __future__ import annotations
 
 import logging
 import uuid
+from collections.abc import Mapping
 from datetime import datetime
-from typing import TYPE_CHECKING, Optional, Sequence, override
+from typing import TYPE_CHECKING, Any, Optional, Sequence, override
 
 from agentle.generations.tracing.contracts.stateful_observability_client import (
     StatefulObservabilityClient,
@@ -497,9 +498,11 @@ class LangfuseObservabilityClient(StatefulObservabilityClient):
         session_id: str | None = None,
         input: object | None = None,
         output: object | None = None,
-        metadata: dict[str, object] | None = None,
+        metadata: Mapping[str, object] | None = None,
         tags: Sequence[str] | None = None,
         timestamp: datetime | None = None,
+        usage_details: Mapping[str, Any] | None = None,
+        cost_details: Mapping[str, Any] | None = None,
     ) -> StatefulObservabilityClient:
         """
         End the current observation in Langfuse.
@@ -548,19 +551,30 @@ class LangfuseObservabilityClient(StatefulObservabilityClient):
             StatefulTraceClient,
         )
 
+        if cost_details and "currency" in cost_details:
+            cost_details = {k: v for k, v in cost_details.items() if k != "currency"}
+
         if self._stateful_client:
             if isinstance(
                 self._stateful_client,
                 (StatefulSpanClient, StatefulGenerationClient),
             ):
                 # For spans and generations, call end()
-                self._stateful_client.end(
-                    name=name,
-                    input=input,
-                    output=output,
-                    metadata=metadata,
-                    end_time=timestamp,
-                )
+                kwargs = {
+                    "name": name,
+                    "input": input,
+                    "output": output,
+                    "metadata": metadata,
+                    "end_time": timestamp,
+                }
+
+                # Only add usage_details and cost_details if they're provided
+                if usage_details:
+                    kwargs["usage"] = usage_details
+                if cost_details:
+                    kwargs["cost_details"] = cost_details
+
+                self._stateful_client.end(**kwargs)  # type: ignore
             elif isinstance(self._stateful_client, StatefulTraceClient):
                 # For traces, call update()
                 self._stateful_client.update(
@@ -597,3 +611,41 @@ class LangfuseObservabilityClient(StatefulObservabilityClient):
             self._logger.debug("Successfully flushed all events to Langfuse")
         except Exception as e:
             self._logger.error(f"Error flushing events to Langfuse: {e}")
+
+    @override
+    async def score_trace(
+        self,
+        *,
+        name: str,
+        value: float | str,
+        trace_id: str | None = None,
+        comment: str | None = None,
+    ) -> None:
+        """
+        Add a score to a trace.
+
+        This method adds a score to a trace in Langfuse, which can be used for
+        evaluating and filtering traces in the UI.
+
+        Args:
+            name: The name of the score (e.g., "trace_success", "response_quality").
+            value: The score value (float for numeric scores, string for categorical).
+            trace_id: Optional trace ID. If not provided, uses the current trace.
+            comment: Optional comment or explanation for the score.
+        """
+        try:
+            target_trace_id = trace_id or self._trace_id
+
+            if not target_trace_id:
+                self._logger.warning("Cannot add score: No trace ID available")
+                return
+
+            if self._client:
+                self._client.score(
+                    trace_id=target_trace_id, name=name, value=value, comment=comment
+                )
+                self._logger.debug(
+                    f"Added score '{name}' with value '{value}' to trace {target_trace_id}"
+                )
+        except Exception as e:
+            self._logger.error(f"Error adding score to trace: {e}")
